from Products.CMFCore import permissions as cmfpermissions
import Products.Archetypes.public as atapi
import config

def _formatPermissionForType(portal_type):
    return "%s: Add %s" % (config.PROJECTNAME, portal_type)

def initialize():
    permissions = {}
    types = atapi.listTypes(config.PROJECTNAME)
    for atype in  types:
        permission = _formatPermissionForType(atype['portal_type'])
        permissions[atype['portal_type']] = permission
        cmfpermissions.setDefaultRoles(permission, ('Manager','Owner'))

    ManageTeamMembership = 'Manage team memberships'
    cmfpermissions.setDefaultRoles(ManageTeamMembership, ('Manager',))

    return permissions

def getQuillsAddPermissions():
    """Return a sequence of all the 'add' permissions for Quills types.
    """
    qtypes = atapi.listTypes(config.PROJECTNAME)
    perms = []
    for atype in  qtypes:
        permission = _formatPermissionForType(atype['portal_type'])
        perms.append(permission)
    return perms

View = cmfpermissions.View
# XXX This next permission is setup in the 'initialize' function above.
# It's probably not the right thing to use, but it's better than nothing.
ViewDrafts = 'Quills: Add WeblogEntry'

# The Quills role
WeblogAuthorRole = 'WeblogAuthorRole'

# Called from Extensions/Install.py. There's probably a better way to do this...
def setupPortalSecurity(portal, out=None):
    portal._addRole(WeblogAuthorRole)
    # Start with the add permissions
    perms = getQuillsAddPermissions()
    # Always need the standard AddPortalContent permission to add anything
    # plus need the DeleteObjects permission to move anything.
    perms.extend( [cmfpermissions.AddPortalContent, cmfpermissions.DeleteObjects] )
    portal.manage_role(WeblogAuthorRole, perms)
    if out is not None:
        out.write('Added %s to the portal.\n' % WeblogAuthorRole)

def unsetupPortalSecurity(portal, out=None):
    portal._delRoles([WeblogAuthorRole,])
    if out is not None:
        out.write('Removed %s from the portal.\n' % WeblogAuthorRole)
