# Zope 3 imports
from zope.interface import implements
from zope.app import zapi
from zope.app.traversing.adapters import Traverser, _marker
from zope.publisher.interfaces.browser import IBrowserRequest
from zope.component import ComponentLookupError

# Five imports
from Products.Five.traversable import FiveTraversable, FakeRequest

# Product imports
from topic import Topic, AuthorsTopic


class SubpathTraverser(Traverser):

    def traverse(self, path, default=_marker, request=None):
        #import pdb; pdb.set_trace()
        clear_name_stack = False
        # path looks like ['topics'], or similar...
        if path[0] in ['topics', 'authors']:
            clear_name_stack = True
            path = path + request['TraversalRequestNameStack']
        obj = super(SubpathTraverser, self).traverse(path, default, request)
        if clear_name_stack:
            request['TraversalRequestNameStack'][:] = []
        return obj


class WeblogTraversable(FiveTraversable):
    """Intercepts traversal for IWeblogs, but only for 'topics'.
    Everything else is left untouched.
    """

    def traverse(self, name, furtherPath):
        # For some reason, when the adapter is constructed, the adapted
        # object is stored as self._subject rather than self.context.
        # I like self.context.
        #import pdb; pdb.set_trace()
        self.context = self._subject
        if name == 'topics':
            topic_class = Topic
        elif name == 'authors':
            topic_class = AuthorsTopic
        else:
            # We ignore it...
            return super(WeblogTraversable, self).traverse(name, furtherPath)
        # Now the guts of it...
        subpath = self.popFurtherPath(furtherPath)
        # furtherPath is now an empty list
        # Find the REQUEST
        REQUEST = getattr(self.context, 'REQUEST', None)
        if not IBrowserRequest.providedBy(REQUEST):
            REQUEST = FakeRequest()
        if len(subpath) == 0:
            return zapi.getView(self.context, name, REQUEST)
        elif len(subpath) == 1:
            furtherPath.append('index.html')
            topic = topic_class(subpath[0]).__of__(self.context)
        elif subpath[-1].startswith('@@'):
            furtherPath.append(subpath[-1])
            topic = topic_class(subpath[:-1]).__of__(self.context)
        else:
            furtherPath.append('index.html')
            topic = topic_class(subpath).__of__(self.context)
        return topic

    def popFurtherPath(self, furtherPath):
        """Empty the furtherPath sequence into a new list.
        """
        subpath = []
        subpath.extend(furtherPath)
        while furtherPath:
            furtherPath.pop()
        return subpath
